package java.util;

import java.util.function.Consumer;
import java.io.InvalidObjectException;
import sun.misc.SharedSecrets;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.function.BiConsumer;
import java.util.function.BiFunction;
import java.util.function.Function;
import java.lang.reflect.Type;
import java.lang.reflect.ParameterizedType;
import java.io.Serializable;

public class HashMap<K, V> extends AbstractMap<K, V> implements Map<K, V>, Cloneable, Serializable
{
    private static final long serialVersionUID = 362498820763181265L;
    static final int DEFAULT_INITIAL_CAPACITY = 16;
    static final int MAXIMUM_CAPACITY = 1073741824;
    static final float DEFAULT_LOAD_FACTOR = 0.75f;
    static final int TREEIFY_THRESHOLD = 8;
    static final int UNTREEIFY_THRESHOLD = 6;
    static final int MIN_TREEIFY_CAPACITY = 64;
    transient HashMap.Node<K, V>[] table;
    transient Set<Map.Entry<K, V>> entrySet;
    transient int size;
    transient int modCount;
    int threshold;
    final float loadFactor;
    
    static final int hash(Object o) {
        int n;
        if (o == null) {
            n = 0;
        }
        else {
            int hashCode = o.hashCode();
            n = (hashCode ^ hashCode >>> 16);
        }
        return n;
    }
    
    static Class<?> comparableClassFor(Object o) {
        if ((o instanceof Comparable)) {
            Class<Object> class1;
            if ((class1 = o.getClass()) == String.class) {
                return class1;
            }
            Type[] genericInterfaces;
            if ((genericInterfaces = class1.getGenericInterfaces()) != null) {
                int i = 0;
                while (i < genericInterfaces.length) {
                    Type type;
                    ParameterizedType parameterizedType;
                    Type[] actualTypeArguments;
                    if ((type = genericInterfaces[i]) instanceof ParameterizedType && (parameterizedType = (ParameterizedType)type).getRawType() == Comparable.class && (actualTypeArguments = parameterizedType.getActualTypeArguments()) != null && actualTypeArguments.length == 1 && actualTypeArguments[0] == class1) {
                        return class1;
                    }
                    ++i;
                }
            }
        }
        return null;
    }
    
    static int compareComparables(Class<?> clazz, Object o, Object o2) {
        return (o2 == null || o2.getClass() != clazz) ? 0 : ((Comparable)o).compareTo(o2);
    }
    
    static final int tableSizeFor(int n) {
        int n2 = n - 1;
        int n3 = n2 | n2 >>> 1;
        int n4 = n3 | n3 >>> 2;
        int n5 = n4 | n4 >>> 4;
        int n6 = n5 | n5 >>> 8;
        int n7 = n6 | n6 >>> 16;
        return (n7 < 0) ? 1 : ((n7 >= 1073741824) ? 1073741824 : (n7 + 1));
    }
    
    public HashMap(int i, float f) {
        if (i < 0) {
            throw new IllegalArgumentException("Illegal initial capacity: " + i);
        }
        if (i > 1073741824) {
            i = 1073741824;
        }
        if (f <= 0.0f || Float.isNaN(f)) {
            throw new IllegalArgumentException("Illegal load factor: " + f);
        }
        this.loadFactor = f;
        this.threshold = tableSizeFor(i);
        return;
    }
    
    public HashMap(int n) {
        this(n, 0.75f);
    }
    
    public HashMap() {
        this.loadFactor = 0.75f;
    }
    
    public HashMap(Map<? extends K, ? extends V> map) {
        this.loadFactor = 0.75f;
        this.putMapEntries(map, false);
    }
    
    final void putMapEntries(Map<? extends K, ? extends V> map, boolean b) {
        int size = map.size();
        if (size > 0) {
            if (this.table == null) {
                float n = (float)size / this.loadFactor + 1.0f;
                int n2 = (n < 1.0737418E9f) ? ((int)n) : 1073741824;
                if (n2 > this.threshold) {
                    this.threshold = tableSizeFor(n2);
                }
            }
            else if (size > this.threshold) {
                this.resize();
            }
            for (Map.Entry entry : map.entrySet()) {
                Object key = entry.getKey();
                this.putVal(hash(key), key, entry.getValue(), false, b);
            }
        }
    }
    
    @Override
    public int size() {
        return this.size;
    }
    
    @Override
    public boolean isEmpty() {
        return this.size == 0;
    }
    
    @Override
    public V get(Object o) {
        HashMap.Node<K, V> node;
        return ((node = this.getNode(hash(o), o)) == null) ? null : node.value;
    }
    
    final HashMap.Node<K, V> getNode(int n, Object o) {
        HashMap.Node[] table;
        int length;
        HashMap.Node<Object, Object> node;
        if ((table = this.table) != null && (length = table.length) > 0 && (node = table[length - 1 & n]) != null) {
            Object key;
            if (node.hash == n && ((key = node.key) == o || (o != null && o.equals(key)))) {
                return node;
            }
            HashMap.Node<Object, Object> node2;
            if ((node2 = node.next) != null) {
                if ((node instanceof HashMap.TreeNode)) {
                    return ((HashMap.TreeNode)node).getTreeNode(n, o);
                }
                Object key2;
                while (node2.hash != n || ((key2 = node2.key) != o && (o == null || !o.equals(key2)))) {
                    if ((node2 = node2.next) == null) {
                        return null;
                    }
                    continue;
                }
                return node2;
            }
        }
        return null;
    }
    
    @Override
    public boolean containsKey(Object o) {
        return this.getNode(hash(o), o) != null;
    }
    
    @Override
    public V put(K k, V v) {
        return this.putVal(hash(k), k, v, false, true);
    }
    
    final V putVal(int n, K k, V value, boolean b, boolean b2) {
        HashMap.Node[] array;
        int n2;
        if ((array = this.table) == null || (n2 = array.length) == 0) {
            n2 = (array = this.resize()).length;
        }
        int n3;
        HashMap.Node<K, V> node;
        if ((node = array[n3 = (n2 - 1 & n)]) == null) {
            array[n3] = this.newNode(n, k, value, null);
        }
        else {
            Label_0222: {
                Object key;
                if (node.hash == n && ((key = node.key) == k || (k != null && k.equals(key)))) {
                }
                else if ((node instanceof HashMap.TreeNode)) {
                    node = ((HashMap.TreeNode)node).putTreeVal(this, array, n, k, value);
                }
                else {
                    int n4;
                    for (n4 = 0; (node = node.next) != null; , ++n4) {
                        if (node.hash == n) {
                            Object key2;
                            if ((key2 = node.key) != k) {
                                if (k != null && k.equals(key2)) {
                                    break Label_0222;
                                }
                            }
                            else {
                                break Label_0222;
                            }
                        }
                    }
                    node.next = this.newNode(n, k, value, null);
                    if (n4 >= 7) {
                        this.treeifyBin(array, n);
                    }
                }
            }
            if (node != null) {
                Object value2 = node.value;
                if (!b || value2 == null) {
                    node.value = value;
                }
                this.afterNodeAccess(node);
                return value2;
            }
        }
        ++this.modCount;
        if (++this.size > this.threshold) {
            this.resize();
        }
        this.afterNodeInsertion(b2);
        return null;
    }
    
    final HashMap.Node<K, V>[] resize() {
        HashMap.Node[] table = this.table;
        int n = (table == null) ? 0 : table.length;
        int threshold = this.threshold;
        int threshold2 = 0;
        int n2;
        if (n > 0) {
            if (n >= 1073741824) {
                this.threshold = Integer.MAX_VALUE;
                return table;
            }
            if ((n2 = n << 1) < 1073741824 && n >= 16) {
                threshold2 = threshold << 1;
            }
        }
        else if (threshold > 0) {
            n2 = threshold;
        }
        else {
            n2 = 16;
            threshold2 = 12;
        }
        if (threshold2 == 0) {
            float n3 = (float)n2 * this.loadFactor;
            threshold2 = ((n2 < 1073741824 && n3 < 1.0737418E9f) ? ((int)n3) : Integer.MAX_VALUE);
        }
        this.threshold = threshold2;
        HashMap.Node[] table2 = (HashMap.Node[])new HashMap.Node[n2];
        this.table = table2;
        if (table != null) {
            for (int i = 0; i < n; ++i) {
                if (table[i] != null) {
                    table[i] = null;
                    HashMap.Node<Object, Object> next;
                    if (next.next == null) {
                        table2[next.hash & n2 - 1] = next;
                    }
                    else if ((next instanceof HashMap.TreeNode)) {
                        ((HashMap.TreeNode)next).split(this, table2, i, n);
                    }
                    else {
                        HashMap.Node<Object, Object> node = null;
                        HashMap.Node<Object, Object> node2 = null;
                        HashMap.Node<Object, Object> node3 = null;
                        HashMap.Node<Object, Object> node4 = null;
                        do {
                            next = next.next;
                            if ((next.hash & n) == 0x0) {
                                if (node2 == null) {
                                    node = next;
                                }
                                else {
                                    node2.next = next;
                                }
                                node2 = next;
                            }
                            else {
                                if (node4 == null) {
                                    node3 = next;
                                }
                                else {
                                    node4.next = next;
                                }
                                node4 = next;
                            }
                        } while (next != null);
                        if (node2 != null) {
                            node2.next = null;
                            table2[i] = node;
                        }
                        if (node4 != null) {
                            node4.next = null;
                            table2[i + n] = node3;
                        }
                    }
                }
            }
            return table2;
        }
        return table2;
    }
    
    final void treeifyBin(HashMap.Node<K, V>[] array, int n) {
        int length;
        if (array == null || (length = array.length) < 64) {
            this.resize();
        }
        else {
            int n2;
            HashMap.Node<K, V> next;
            if ((next = array[n2 = (length - 1 & n)]) != null) {
                HashMap.TreeNode<Object, Object> treeNode = null;
                HashMap.TreeNode<Object, Object> prev = null;
                do {
                    HashMap.TreeNode<K, V> replacementTreeNode = this.replacementTreeNode(next, null);
                    if (prev == null) {
                        treeNode = replacementTreeNode;
                    }
                    else {
                        replacementTreeNode.prev = prev;
                        prev.next = replacementTreeNode;
                    }
                    prev = replacementTreeNode;
                } while ((next = next.next) != null);
                if ((array[n2] = treeNode) != null) {
                    treeNode.treeify(array);
                }
            }
        }
    }
    
    @Override
    public void putAll(Map<? extends K, ? extends V> map) {
        this.putMapEntries(map, true);
    }
    
    @Override
    public V remove(Object o) {
        HashMap.Node<K, V> removeNode;
        return ((removeNode = this.removeNode(hash(o), o, null, false, true)) == null) ? null : removeNode.value;
    }
    
    final HashMap.Node<K, V> removeNode(int n, Object o, Object o2, boolean b, boolean b2) {
        HashMap.Node[] table;
        int length;
        int n2;
        HashMap.Node<Object, Object> node;
        if ((table = this.table) != null && (length = table.length) > 0 && (node = table[n2 = (length - 1 & n)]) != null) {
            HashMap.Node<K, V> treeNode = null;
            Label_0171: {
                Object key;
                if (node.hash == n && ((key = node.key) == o || (o != null && o.equals(key)))) {
                    treeNode = node;
                }
                else {
                    HashMap.Node<Object, Object> node2;
                    if ((node2 = node.next) != null) {
                        if ((node instanceof HashMap.TreeNode)) {
                            treeNode = ((HashMap.TreeNode)node).getTreeNode(n, o);
                        }
                        else {
                            Object key2;
                            while (node2.hash != n || ((key2 = node2.key) != o && (o == null || !o.equals(key2)))) {
                                node = node2;
                                if ((node2 = node2.next) == null) {
                                    break Label_0171;
                                }
                                continue;
                            }
                            treeNode = node2;
                        }
                    }
                }
            }
            Object value;
            if (treeNode != null && (!b || (value = treeNode.value) == o2 || (o2 != null && o2.equals(value)))) {
                if ((treeNode instanceof HashMap.TreeNode)) {
                    ((HashMap.TreeNode)treeNode).removeTreeNode(this, table, b2);
                }
                else if (treeNode == node) {
                    table[n2] = treeNode.next;
                }
                else {
                    node.next = treeNode.next;
                }
                ++this.modCount;
                --this.size;
                this.afterNodeRemoval(treeNode);
                return treeNode;
            }
        }
        return null;
    }
    
    @Override
    public void clear() {
        ++this.modCount;
        HashMap.Node[] table;
        if ((table = this.table) != null && this.size > 0) {
            this.size = 0;
            for (int i = 0; i < table.length; ++i) {
                table[i] = null;
            }
        }
    }
    
    @Override
    public boolean containsValue(Object o) {
        HashMap.Node[] table;
        if ((table = this.table) != null && this.size > 0) {
            for (int i = 0; i < table.length; ++i) {
                HashMap.Node<Object, Object> next = table[i];
                while (next != null) {
                    Object value;
                    if ((value = next.value) == o || (o != null && o.equals(value))) {
                        return true;
                    }
                    next = next.next;
                }
            }
        }
        return false;
    }
    
    @Override
    public Set<K> keySet() {
        Set<Object> keySet = this.keySet;
        if (keySet == null) {
            keySet = new HashMap.KeySet();
            this.keySet = keySet;
            return keySet;
        }
        return keySet;
    }
    
    @Override
    public Collection<V> values() {
        Collection<Object> values = this.values;
        if (values == null) {
            values = new HashMap.Values();
            this.values = values;
            return values;
        }
        return values;
    }
    
    @Override
    public Set<Map.Entry<K, V>> entrySet() {
        Set<Object> entrySet;
        return ((entrySet = this.entrySet) == null) ? (this.entrySet = new HashMap.EntrySet()) : entrySet;
    }
    
    @Override
    public V getOrDefault(Object o, V v) {
        HashMap.Node<K, V> node;
        return ((node = this.getNode(hash(o), o)) == null) ? v : node.value;
    }
    
    @Override
    public V putIfAbsent(K k, V v) {
        return this.putVal(hash(k), k, v, true, true);
    }
    
    @Override
    public boolean remove(Object o, Object o2) {
        return this.removeNode(hash(o), o, o2, true, true) != null;
    }
    
    @Override
    public boolean replace(K k, V obj, V value) {
        HashMap.Node<K, V> node;
        Object value2;
        if ((node = this.getNode(hash(k), k)) != null && ((value2 = node.value) == obj || (value2 != null && value2.equals(obj)))) {
            node.value = value;
            this.afterNodeAccess(node);
            return true;
        }
        return false;
    }
    
    @Override
    public V replace(K k, V value) {
        HashMap.Node<K, V> node;
        if ((node = this.getNode(hash(k), k)) != null) {
            Object value2 = node.value;
            node.value = value;
            this.afterNodeAccess(node);
            return value2;
        }
        return null;
    }
    
    @Override
    public V computeIfAbsent(K k, Function<? super K, ? extends V> function) {
        if (function == null) {
            throw new NullPointerException();
        }
        int hash = hash(k);
        int n = 0;
        HashMap.TreeNode treeNode = null;
        HashMap.Node<K, V> treeNode2 = null;
        HashMap.Node[] array;
        int n2;
        if (this.size > this.threshold || (array = this.table) == null || (n2 = array.length) == 0) {
            n2 = (array = this.resize()).length;
        }
        int n3;
        HashMap.Node<K, V> next;
        if ((next = array[n3 = (n2 - 1 & hash)]) != null) {
            Label_0169: {
                if ((next instanceof HashMap.TreeNode)) {
                    treeNode2 = (treeNode = (HashMap.TreeNode)next).getTreeNode(hash, k);
                }
                else {
                    Object key;
                    while (next.hash != hash || ((key = next.key) != k && (k == null || !k.equals(key)))) {
                        ++n;
                        if ((next = next.next) == null) {
                            break Label_0169;
                        }
                        continue;
                    }
                    treeNode2 = next;
                }
            }
            Object value;
            if (treeNode2 != null && (value = treeNode2.value) != null) {
                this.afterNodeAccess(treeNode2);
                return value;
            }
        }
        Object apply = function.apply(k);
        if (apply == null) {
            return null;
        }
        if (treeNode2 != null) {
            treeNode2.value = apply;
            this.afterNodeAccess(treeNode2);
            return apply;
        }
        if (treeNode != null) {
            treeNode.putTreeVal(this, array, hash, k, apply);
        }
        else {
            array[n3] = this.newNode(hash, k, apply, next);
            if (n >= 7) {
                this.treeifyBin(array, hash);
            }
        }
        ++this.modCount;
        ++this.size;
        this.afterNodeInsertion(true);
        return apply;
    }
    
    @Override
    public V computeIfPresent(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
        if (biFunction == null) {
            throw new NullPointerException();
        }
        int hash = hash(k);
        HashMap.Node<K, V> node;
        Object value;
        if ((node = this.getNode(hash, k)) != null && (value = node.value) != null) {
            Object apply = biFunction.apply(k, value);
            if (apply != null) {
                node.value = apply;
                this.afterNodeAccess(node);
                return apply;
            }
            this.removeNode(hash, k, null, false, true);
        }
        return null;
    }
    
    @Override
    public V compute(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
        if (biFunction == null) {
            throw new NullPointerException();
        }
        int hash = hash(k);
        int n = 0;
        HashMap.TreeNode treeNode = null;
        HashMap.Node<K, V> treeNode2 = null;
        HashMap.Node[] array;
        int n2;
        if (this.size > this.threshold || (array = this.table) == null || (n2 = array.length) == 0) {
            n2 = (array = this.resize()).length;
        }
        int n3;
        HashMap.Node<K, V> next;
        Label_0169: {
            if ((next = array[n3 = (n2 - 1 & hash)]) != null) {
                if ((next instanceof HashMap.TreeNode)) {
                    treeNode2 = (treeNode = (HashMap.TreeNode)next).getTreeNode(hash, k);
                }
                else {
                    Object key;
                    while (next.hash != hash || ((key = next.key) != k && (k == null || !k.equals(key)))) {
                        ++n;
                        if ((next = next.next) == null) {
                            break Label_0169;
                        }
                        continue;
                    }
                    treeNode2 = next;
                }
            }
        }
        Object apply = biFunction.apply(k, (treeNode2 == null) ? null : treeNode2.value);
        if (treeNode2 != null) {
            if (apply != null) {
                treeNode2.value = apply;
                this.afterNodeAccess(treeNode2);
            }
            else {
                this.removeNode(hash, k, null, false, true);
            }
        }
        else if (apply != null) {
            if (treeNode != null) {
                treeNode.putTreeVal(this, array, hash, k, apply);
            }
            else {
                array[n3] = this.newNode(hash, k, apply, next);
                if (n >= 7) {
                    this.treeifyBin(array, hash);
                }
            }
            ++this.modCount;
            ++this.size;
            this.afterNodeInsertion(true);
        }
        return apply;
    }
    
    @Override
    public V merge(K k, V v, BiFunction<? super V, ? super V, ? extends V> biFunction) {
        if (v == null) {
            throw new NullPointerException();
        }
        if (biFunction == null) {
            throw new NullPointerException();
        }
        int hash = hash(k);
        int n = 0;
        HashMap.TreeNode treeNode = null;
        HashMap.Node<K, V> treeNode2 = null;
        HashMap.Node[] array;
        int n2;
        if (this.size > this.threshold || (array = this.table) == null || (n2 = array.length) == 0) {
            n2 = (array = this.resize()).length;
        }
        int n3;
        HashMap.Node<K, V> next;
        Label_0185: {
            if ((next = array[n3 = (n2 - 1 & hash)]) != null) {
                if ((next instanceof HashMap.TreeNode)) {
                    treeNode2 = (treeNode = (HashMap.TreeNode)next).getTreeNode(hash, k);
                }
                else {
                    Object key;
                    while (next.hash != hash || ((key = next.key) != k && (k == null || !k.equals(key)))) {
                        ++n;
                        if ((next = next.next) == null) {
                            break Label_0185;
                        }
                        continue;
                    }
                    treeNode2 = next;
                }
            }
        }
        if (treeNode2 != null) {
            Object apply;
            if (treeNode2.value != null) {
                apply = biFunction.apply(treeNode2.value, v);
            }
            else {
                apply = v;
            }
            if (apply != null) {
                treeNode2.value = apply;
                this.afterNodeAccess(treeNode2);
                return apply;
            }
            this.removeNode(hash, k, null, false, true);
            return apply;
        }
        else if (v != null) {
            if (treeNode != null) {
                treeNode.putTreeVal(this, array, hash, k, v);
            }
            else {
                array[n3] = this.newNode(hash, k, v, next);
                if (n >= 7) {
                    this.treeifyBin(array, hash);
                }
            }
            ++this.modCount;
            ++this.size;
            this.afterNodeInsertion(true);
            return v;
        }
        else {
            return v;
        }
    }
    
    @Override
    public void forEach(BiConsumer<? super K, ? super V> biConsumer) {
        if (biConsumer == null) {
            throw new NullPointerException();
        }
        HashMap.Node[] table;
        if (this.size > 0 && (table = this.table) != null) {
            int modCount = this.modCount;
            for (int i = 0; i < table.length; ++i) {
                for (HashMap.Node next = table[i]; next != null; next = next.next) {
                    biConsumer.accept(next.key, next.value);
                }
            }
            if (this.modCount != modCount) {
                throw new ConcurrentModificationException();
            }
        }
        return;
    }
    
    @Override
    public void replaceAll(BiFunction<? super K, ? super V, ? extends V> biFunction) {
        if (biFunction == null) {
            throw new NullPointerException();
        }
        HashMap.Node[] table;
        if (this.size > 0 && (table = this.table) != null) {
            int modCount = this.modCount;
            for (int i = 0; i < table.length; ++i) {
                for (HashMap.Node next = table[i]; next != null; next = next.next) {
                    next.value = biFunction.apply(next.key, next.value);
                }
            }
            if (this.modCount != modCount) {
                throw new ConcurrentModificationException();
            }
        }
        return;
    }
    
    public Object clone() {
        HashMap<Object, Object> hashMap;
        try {
            hashMap = (HashMap)super.clone();
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
        hashMap.reinitialize();
        hashMap.putMapEntries(this, false);
        return hashMap;
    }
    
    final float loadFactor() {
        return this.loadFactor;
    }
    
    final int capacity() {
        return (this.table != null) ? this.table.length : ((this.threshold > 0) ? this.threshold : 16);
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        int capacity = this.capacity();
        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeInt(capacity);
        objectOutputStream.writeInt(this.size);
        this.internalWriteEntries(objectOutputStream);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.reinitialize();
        if (this.loadFactor <= 0.0f || Float.isNaN(this.loadFactor)) {
            throw new InvalidObjectException("Illegal load factor: " + this.loadFactor);
        }
        objectInputStream.readInt();
        int int1 = objectInputStream.readInt();
        if (int1 < 0) {
            throw new InvalidObjectException("Illegal mappings count: " + int1);
        }
        if (int1 > 0) {
            float min = Math.min(Math.max(0.25f, this.loadFactor), 4.0f);
            float n = (float)int1 / min + 1.0f;
            int n2 = (n < 16.0f) ? 16 : ((n >= 1.0737418E9f) ? 1073741824 : tableSizeFor((int)n));
            float n3 = (float)n2 * min;
            this.threshold = ((n2 < 1073741824 && n3 < 1.0737418E9f) ? ((int)n3) : Integer.MAX_VALUE);
            SharedSecrets.getJavaOISAccess().checkArray(objectInputStream, Map.Entry[].class, n2);
            this.table = (HashMap.Node[])new HashMap.Node[n2];
            for (int i = 0; i < int1; ++i) {
                Object object = objectInputStream.readObject();
                this.putVal(hash(object), object, objectInputStream.readObject(), false, false);
            }
        }
        return;
    }
    
    HashMap.Node<K, V> newNode(int n, K k, V v, HashMap.Node<K, V> node) {
        return new HashMap.Node(n, k, v, node);
    }
    
    HashMap.Node<K, V> replacementNode(HashMap.Node<K, V> node, HashMap.Node<K, V> node2) {
        return new HashMap.Node(node.hash, node.key, node.value, node2);
    }
    
    HashMap.TreeNode<K, V> newTreeNode(int n, K k, V v, HashMap.Node<K, V> node) {
        return new HashMap.TreeNode(n, k, v, node);
    }
    
    HashMap.TreeNode<K, V> replacementTreeNode(HashMap.Node<K, V> node, HashMap.Node<K, V> node2) {
        return new HashMap.TreeNode(node.hash, node.key, node.value, node2);
    }
    
    void reinitialize() {
        this.table = null;
        this.entrySet = null;
        this.keySet = null;
        this.values = null;
        this.modCount = 0;
        this.threshold = 0;
        this.size = 0;
    }
    
    void afterNodeAccess(HashMap.Node<K, V> node) {
    }
    
    void afterNodeInsertion(boolean b) {
    }
    
    void afterNodeRemoval(HashMap.Node<K, V> node) {
    }
    
    void internalWriteEntries(ObjectOutputStream objectOutputStream) throws IOException {
        HashMap.Node[] table;
        if (this.size > 0 && (table = this.table) != null) {
            for (int i = 0; i < table.length; ++i) {
                for (HashMap.Node next = table[i]; next != null; next = next.next) {
                    objectOutputStream.writeObject(next.key);
                    objectOutputStream.writeObject(next.value);
                }
            }
        }
    }
    
    static class Node<K, V> implements Map.Entry<K, V>
    {
        final int hash;
        final K key;
        V value;
        HashMap.Node<K, V> next;
        
        Node(int hash, K key, V value, HashMap.Node<K, V> next) {
            this.hash = hash;
            this.key = key;
            this.value = value;
            this.next = next;
        }
        
        @Override
        public final K getKey() {
            return this.key;
        }
        
        @Override
        public final V getValue() {
            return this.value;
        }
        
        @Override
        public final String toString() {
            return this.key + "=" + this.value;
        }
        
        @Override
        public final int hashCode() {
            return Objects.hashCode(this.key) ^ Objects.hashCode(this.value);
        }
        
        @Override
        public final V setValue(V value) {
            Object value2 = this.value;
            this.value = value;
            return value2;
        }
        
        @Override
        public final boolean equals(Object o) {
            if (o == this) {
                return true;
            }
            if ((o instanceof Map.Entry)) {
                Map.Entry<Object, Object> entry = (Map.Entry)o;
                if (Objects.equals(this.key, entry.getKey()) && Objects.equals(this.value, entry.getValue())) {
                    return true;
                }
            }
            return false;
        }
    }
    
    final class KeySet extends AbstractSet<K>
    {
        @Override
        public final int size() {
            return HashMap.this.size;
        }
        
        @Override
        public final void clear() {
            HashMap.this.clear();
        }
        
        @Override
        public final Iterator<K> iterator() {
            return new HashMap.KeyIterator();
        }
        
        @Override
        public final boolean contains(Object o) {
            return HashMap.this.containsKey(o);
        }
        
        @Override
        public final boolean remove(Object o) {
            return HashMap.this.removeNode(HashMap.hash(o), o, null, false, true) != null;
        }
        
        @Override
        public final Spliterator<K> spliterator() {
            return new HashMap.KeySpliterator(HashMap.this, 0, -1, 0, 0);
        }
        
        @Override
        public final void forEach(Consumer<? super K> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            HashMap.Node[] table;
            if (HashMap.this.size > 0 && (table = HashMap.this.table) != null) {
                int modCount = HashMap.this.modCount;
                for (int i = 0; i < table.length; ++i) {
                    for (HashMap.Node next = table[i]; next != null; next = next.next) {
                        consumer.accept(next.key);
                    }
                }
                if (HashMap.this.modCount != modCount) {
                    throw new ConcurrentModificationException();
                }
            }
            return;
        }
    }
    
    final class Values extends AbstractCollection<V>
    {
        @Override
        public final int size() {
            return HashMap.this.size;
        }
        
        @Override
        public final void clear() {
            HashMap.this.clear();
        }
        
        @Override
        public final Iterator<V> iterator() {
            return new HashMap.ValueIterator();
        }
        
        @Override
        public final boolean contains(Object o) {
            return HashMap.this.containsValue(o);
        }
        
        @Override
        public final Spliterator<V> spliterator() {
            return new HashMap.ValueSpliterator(HashMap.this, 0, -1, 0, 0);
        }
        
        @Override
        public final void forEach(Consumer<? super V> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            HashMap.Node[] table;
            if (HashMap.this.size > 0 && (table = HashMap.this.table) != null) {
                int modCount = HashMap.this.modCount;
                for (int i = 0; i < table.length; ++i) {
                    for (HashMap.Node next = table[i]; next != null; next = next.next) {
                        consumer.accept(next.value);
                    }
                }
                if (HashMap.this.modCount != modCount) {
                    throw new ConcurrentModificationException();
                }
            }
            return;
        }
    }
    
    final class EntrySet extends AbstractSet<Map.Entry<K, V>>
    {
        @Override
        public final int size() {
            return HashMap.this.size;
        }
        
        @Override
        public final void clear() {
            HashMap.this.clear();
        }
        
        @Override
        public final Iterator<Map.Entry<K, V>> iterator() {
            return new HashMap.EntryIterator();
        }
        
        @Override
        public final boolean contains(Object o) {
            if (!(o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            Object key = entry.getKey();
            HashMap.Node<Object, Object> node = HashMap.this.getNode(HashMap.hash(key), key);
            return node != null && node.equals(entry);
        }
        
        @Override
        public final boolean remove(Object o) {
            if ((o instanceof Map.Entry)) {
                Map.Entry<Object, Object> entry = (Map.Entry)o;
                Object key = entry.getKey();
                return HashMap.this.removeNode(HashMap.hash(key), key, entry.getValue(), true, true) != null;
            }
            return false;
        }
        
        @Override
        public final Spliterator<Map.Entry<K, V>> spliterator() {
            return new HashMap.EntrySpliterator(HashMap.this, 0, -1, 0, 0);
        }
        
        @Override
        public final void forEach(Consumer<? super Map.Entry<K, V>> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            HashMap.Node[] table;
            if (HashMap.this.size > 0 && (table = HashMap.this.table) != null) {
                int modCount = HashMap.this.modCount;
                for (int i = 0; i < table.length; ++i) {
                    for (HashMap.Node next = table[i]; next != null; next = next.next) {
                        consumer.accept(next);
                    }
                }
                if (HashMap.this.modCount != modCount) {
                    throw new ConcurrentModificationException();
                }
            }
            return;
        }
    }
    
    abstract class HashIterator
    {
        HashMap.Node<K, V> next;
        HashMap.Node<K, V> current;
        int expectedModCount;
        int index;
        
        HashIterator() {
            this.expectedModCount = HashMap.this.modCount;
            HashMap.Node[] table = HashMap.this.table;
            HashMap.Node<K, V> node = null;
            this.next = node;
            this.current = node;
            this.index = 0;
            if (table != null && HashMap.this.size > 0) {
                while (this.index < table.length && (this.next = table[this.index++]) == null) {}
            }
        }
        
        public final boolean hasNext() {
            return this.next != null;
        }
        
        final HashMap.Node<K, V> nextNode() {
            HashMap.Node<Object, Object> next = this.next;
            if (HashMap.this.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            if (next == null) {
                throw new NoSuchElementException();
            }
            this.current = next;
            HashMap.Node<K, V> next2 = next.next;
            this.next = next2;
            HashMap.Node[] table;
            if (next2 == null && (table = HashMap.this.table) != null) {
                while (this.index < table.length && (this.next = table[this.index++]) == null) {}
                return next;
            }
            return next;
        }
        
        public final void remove() {
            HashMap.Node<Object, Object> current = this.current;
            if (current == null) {
                throw new IllegalStateException();
            }
            if (HashMap.this.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            this.current = null;
            Object key = current.key;
            HashMap.this.removeNode(HashMap.hash(key), key, null, false, false);
            this.expectedModCount = HashMap.this.modCount;
            return;
        }
    }
    
    final class KeyIterator extends HashMap.HashIterator implements Iterator<K>
    {
        @Override
        public final K next() {
            return this.nextNode().key;
        }
    }
    
    final class ValueIterator extends HashMap.HashIterator implements Iterator<V>
    {
        @Override
        public final V next() {
            return this.nextNode().value;
        }
    }
    
    final class EntryIterator extends HashMap.HashIterator implements Iterator<Map.Entry<K, V>>
    {
        @Override
        public final Map.Entry<K, V> next() {
            return this.nextNode();
        }
    }
    
    static class HashMapSpliterator<K, V>
    {
        final HashMap<K, V> map;
        HashMap.Node<K, V> current;
        int index;
        int fence;
        int est;
        int expectedModCount;
        
        HashMapSpliterator(HashMap<K, V> map, int index, int fence, int est, int expectedModCount) {
            this.map = map;
            this.index = index;
            this.fence = fence;
            this.est = est;
            this.expectedModCount = expectedModCount;
        }
        
        final int getFence() {
            int fence;
            if ((fence = this.fence) < 0) {
                HashMap<Object, Object> map = this.map;
                this.est = map.size;
                this.expectedModCount = map.modCount;
                HashMap.Node[] table = map.table;
                int fence2 = (table == null) ? 0 : table.length;
                this.fence = fence2;
                fence = fence2;
            }
            return fence;
        }
        
        public final long estimateSize() {
            this.getFence();
            return (long)this.est;
        }
    }
    
    static final class KeySpliterator<K, V> extends HashMap.HashMapSpliterator<K, V> implements Spliterator<K>
    {
        KeySpliterator(HashMap<K, V> hashMap, int n, int n2, int n3, int n4) {
            super(hashMap, n, n2, n3, n4);
        }
        
        @Override
        public HashMap.KeySpliterator<K, V> trySplit() {
            int fence = this.getFence();
            int index = this.index;
            int n = index + fence >>> 1;
            if (index >= n || this.current != null) {
                return null;
            }
            HashMap<K, V> map;
            int n2;
            int index2;
            HashMap.KeySpliterator keySpliterator = new HashMap.KeySpliterator(map, n2, index2, this.est >>>= 1, this.expectedModCount);
            map = this.map;
            n2 = index;
            index2 = n;
            this.index = index2;
            return keySpliterator;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super K> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            HashMap<Object, Object> map = this.map;
            HashMap.Node[] table = map.table;
            int fence;
            int expectedModCount;
            if ((fence = this.fence) < 0) {
                int modCount = map.modCount;
                this.expectedModCount = modCount;
                expectedModCount = modCount;
                int fence2 = (table == null) ? 0 : table.length;
                this.fence = fence2;
                fence = fence2;
            }
            else {
                expectedModCount = this.expectedModCount;
            }
            int index;
            if (table != null && table.length >= fence && (index = this.index) >= 0) {
                int n = index;
                int index2 = fence;
                this.index = index2;
                if (n < index2 || this.current != null) {
                    HashMap.Node<Object, Object> node = this.current;
                    this.current = null;
                    do {
                        if (node == null) {
                            node = table[index++];
                        }
                        else {
                            consumer.accept(node.key);
                            node = node.next;
                        }
                    } while (node != null || index < fence);
                    if (map.modCount != expectedModCount) {
                        throw new ConcurrentModificationException();
                    }
                }
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super K> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            HashMap.Node[] table = this.map.table;
            int fence;
            if (table != null && table.length >= (fence = this.getFence()) && this.index >= 0) {
                while (this.current != null || this.index < fence) {
                    if (this.current == null) {
                        this.current = table[this.index++];
                    }
                    else {
                        Object key = this.current.key;
                        this.current = this.current.next;
                        consumer.accept(key);
                        if (this.map.modCount != this.expectedModCount) {
                            throw new ConcurrentModificationException();
                        }
                        return true;
                    }
                }
            }
            return false;
        }
        
        @Override
        public int characteristics() {
            return ((this.fence < 0 || this.est == this.map.size) ? 64 : 0) | 0x1;
        }
    }
    
    static final class ValueSpliterator<K, V> extends HashMap.HashMapSpliterator<K, V> implements Spliterator<V>
    {
        ValueSpliterator(HashMap<K, V> hashMap, int n, int n2, int n3, int n4) {
            super(hashMap, n, n2, n3, n4);
        }
        
        @Override
        public HashMap.ValueSpliterator<K, V> trySplit() {
            int fence = this.getFence();
            int index = this.index;
            int n = index + fence >>> 1;
            if (index >= n || this.current != null) {
                return null;
            }
            HashMap<K, V> map;
            int n2;
            int index2;
            HashMap.ValueSpliterator valueSpliterator = new HashMap.ValueSpliterator(map, n2, index2, this.est >>>= 1, this.expectedModCount);
            map = this.map;
            n2 = index;
            index2 = n;
            this.index = index2;
            return valueSpliterator;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super V> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            HashMap<Object, Object> map = this.map;
            HashMap.Node[] table = map.table;
            int fence;
            int expectedModCount;
            if ((fence = this.fence) < 0) {
                int modCount = map.modCount;
                this.expectedModCount = modCount;
                expectedModCount = modCount;
                int fence2 = (table == null) ? 0 : table.length;
                this.fence = fence2;
                fence = fence2;
            }
            else {
                expectedModCount = this.expectedModCount;
            }
            int index;
            if (table != null && table.length >= fence && (index = this.index) >= 0) {
                int n = index;
                int index2 = fence;
                this.index = index2;
                if (n < index2 || this.current != null) {
                    HashMap.Node<Object, Object> node = this.current;
                    this.current = null;
                    do {
                        if (node == null) {
                            node = table[index++];
                        }
                        else {
                            consumer.accept(node.value);
                            node = node.next;
                        }
                    } while (node != null || index < fence);
                    if (map.modCount != expectedModCount) {
                        throw new ConcurrentModificationException();
                    }
                }
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super V> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            HashMap.Node[] table = this.map.table;
            int fence;
            if (table != null && table.length >= (fence = this.getFence()) && this.index >= 0) {
                while (this.current != null || this.index < fence) {
                    if (this.current == null) {
                        this.current = table[this.index++];
                    }
                    else {
                        Object value = this.current.value;
                        this.current = this.current.next;
                        consumer.accept(value);
                        if (this.map.modCount != this.expectedModCount) {
                            throw new ConcurrentModificationException();
                        }
                        return true;
                    }
                }
            }
            return false;
        }
        
        @Override
        public int characteristics() {
            return (this.fence < 0 || this.est == this.map.size) ? 64 : 0;
        }
    }
    
    static final class EntrySpliterator<K, V> extends HashMap.HashMapSpliterator<K, V> implements Spliterator<Map.Entry<K, V>>
    {
        EntrySpliterator(HashMap<K, V> hashMap, int n, int n2, int n3, int n4) {
            super(hashMap, n, n2, n3, n4);
        }
        
        @Override
        public HashMap.EntrySpliterator<K, V> trySplit() {
            int fence = this.getFence();
            int index = this.index;
            int n = index + fence >>> 1;
            if (index >= n || this.current != null) {
                return null;
            }
            HashMap<K, V> map;
            int n2;
            int index2;
            HashMap.EntrySpliterator entrySpliterator = new HashMap.EntrySpliterator(map, n2, index2, this.est >>>= 1, this.expectedModCount);
            map = this.map;
            n2 = index;
            index2 = n;
            this.index = index2;
            return entrySpliterator;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super Map.Entry<K, V>> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            HashMap<Object, Object> map = this.map;
            HashMap.Node[] table = map.table;
            int fence;
            int expectedModCount;
            if ((fence = this.fence) < 0) {
                int modCount = map.modCount;
                this.expectedModCount = modCount;
                expectedModCount = modCount;
                int fence2 = (table == null) ? 0 : table.length;
                this.fence = fence2;
                fence = fence2;
            }
            else {
                expectedModCount = this.expectedModCount;
            }
            int index;
            if (table != null && table.length >= fence && (index = this.index) >= 0) {
                int n = index;
                int index2 = fence;
                this.index = index2;
                if (n < index2 || this.current != null) {
                    HashMap.Node<Object, Object> node = this.current;
                    this.current = null;
                    do {
                        if (node == null) {
                            node = table[index++];
                        }
                        else {
                            consumer.accept(node);
                            node = node.next;
                        }
                    } while (node != null || index < fence);
                    if (map.modCount != expectedModCount) {
                        throw new ConcurrentModificationException();
                    }
                }
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super Map.Entry<K, V>> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            HashMap.Node[] table = this.map.table;
            int fence;
            if (table != null && table.length >= (fence = this.getFence()) && this.index >= 0) {
                while (this.current != null || this.index < fence) {
                    if (this.current == null) {
                        this.current = table[this.index++];
                    }
                    else {
                        HashMap.Node<Object, Object> current = this.current;
                        this.current = this.current.next;
                        consumer.accept(current);
                        if (this.map.modCount != this.expectedModCount) {
                            throw new ConcurrentModificationException();
                        }
                        return true;
                    }
                }
            }
            return false;
        }
        
        @Override
        public int characteristics() {
            return ((this.fence < 0 || this.est == this.map.size) ? 64 : 0) | 0x1;
        }
    }
    
    static final class TreeNode<K, V> extends LinkedHashMap.Entry<K, V>
    {
        HashMap.TreeNode<K, V> parent;
        HashMap.TreeNode<K, V> left;
        HashMap.TreeNode<K, V> right;
        HashMap.TreeNode<K, V> prev;
        boolean red;
        
        TreeNode(int n, K k, V v, HashMap.Node<K, V> node) {
            super(n, k, v, node);
        }
        
        final HashMap.TreeNode<K, V> root() {
            HashMap.TreeNode<Object, Object> parent;
            while ((parent = this.parent) != null) {
                this = parent;
            }
            return this;
        }
        
        static <K, V> void moveRootToFront(HashMap.Node<K, V>[] array, HashMap.TreeNode<K, V> prev) {
            int length;
            if (prev != null && array != null && (length = array.length) > 0) {
                int n = length - 1 & prev.hash;
                HashMap.TreeNode<Object, Object> next = (HashMap.TreeNode)array[n];
                if (prev != next) {
                    array[n] = prev;
                    HashMap.TreeNode<Object, Object> prev2 = prev.prev;
                    HashMap.Node<Object, Object> next2;
                    if ((next2 = prev.next) != null) {
                        ((HashMap.TreeNode)next2).prev = prev2;
                    }
                    if (prev2 != null) {
                        prev2.next = next2;
                    }
                    if (next != null) {
                        next.prev = prev;
                    }
                    prev.next = next;
                    prev.prev = null;
                }
                assert !(!checkInvariants(prev));
            }
        }
        
        final HashMap.TreeNode<K, V> find(int p0, Object p1, Class<?> p2) {
            // Bytecode:
            // 
            //     1: astore          4
            //     3: aload           4
            //     5: getfield        java/util/HashMap$TreeNode.left:Ljava/util/HashMap$TreeNode;
            //     8: astore          8
            //    10: aload           4
            //    12: getfield        java/util/HashMap$TreeNode.right:Ljava/util/HashMap$TreeNode;
            //    15: astore          9
            //    17: aload           4
            //    19: getfield        java/util/HashMap$TreeNode.hash:I
            //    22: dup            
            //    23: istore          5
            //    25: iload_1        
            //    26: if_icmple       36
            //    29: aload           8
            //    31: astore          4
            //    33: goto            165
            //    36: iload           5
            //    38: iload_1        
            //    39: if_icmpge       49
            //    42: aload           9
            //    44: astore          4
            //    46: goto            165
            //    49: aload           4
            //    51: getfield        java/util/HashMap$TreeNode.key:Ljava/lang/Object;
            //    54: dup            
            //    55: astore          7
            //    57: aload_2        
            //    58: if_acmpeq       74
            //    61: aload_2        
            //    62: ifnull          77
            //    65: aload_2        
            //    66: aload           7
            //    68: invokevirtual   java/lang/Object.equals:(Ljava/lang/Object;)Z
            //    71: ifeq            77
            //    74: aload           4
            //    76: areturn        
            //    77: aload           8
            //    79: ifnonnull       89
            //    82: aload           9
            //    84: astore          4
            //    86: goto            165
            //    89: aload           9
            //    91: ifnonnull       101
            //    94: aload           8
            //    96: astore          4
            //    98: goto            165
            //   101: aload_3        
            //   102: ifnonnull       114
            //   105: aload_2        
            //   106: invokestatic    java/util/HashMap.comparableClassFor:(Ljava/lang/Object;)Ljava/lang/Class;
            //   109: dup            
            //   110: astore_3       
            //   111: ifnull          144
            //   114: aload_3        
            //   115: aload_2        
            //   116: aload           7
            //   118: invokestatic    java/util/HashMap.compareComparables:(Ljava/lang/Class;Ljava/lang/Object;Ljava/lang/Object;)I
            //   121: dup            
            //   122: istore          6
            //   124: ifeq            144
            //   127: iload           6
            //   129: ifge            137
            //   132: aload           8
            //   134: goto            139
            //   137: aload           9
            //   139: astore          4
            //   141: goto            165
            //   144: aload           9
            //   146: iload_1        
            //   147: aload_2        
            //   148: aload_3        
            //   149: invokevirtual   java/util/HashMap$TreeNode.find:(ILjava/lang/Object;Ljava/lang/Class;)Ljava/util/HashMap$TreeNode;
            //   152: dup            
            //   153: astore          10
            //   155: ifnull          161
            //   158: aload           10
            //   160: areturn        
            //   161: aload           8
            //   163: astore          4
            //   165: aload           4
            //   167: ifnonnull       3
            //   170: aconst_null    
            //   171: areturn        
            //    Signature:
            //  (ILjava/lang/Object;Ljava/lang/Class<*>;)Ljava/util/HashMap$TreeNode<TK;TV;>;
            //    StackMapTable: 00 0D FC 00 03 07 00 49 FF 00 20 00 0A 07 00 49 01 07 00 45 07 00 44 07 00 49 01 00 00 07 00 49 07 00 49 00 00 0C FF 00 18 00 0A 07 00 49 01 07 00 45 07 00 44 07 00 49 01 00 07 00 45 07 00 49 07 00 49 00 00 02 0B 0B 0C FF 00 16 00 0A 07 00 49 01 07 00 45 07 00 44 07 00 49 01 01 07 00 45 07 00 49 07 00 49 00 00 41 07 00 49 FF 00 04 00 0A 07 00 49 01 07 00 45 07 00 44 07 00 49 01 00 07 00 45 07 00 49 07 00 49 00 00 FC 00 10 07 00 49 FF 00 03 00 05 07 00 49 01 07 00 45 07 00 44 07 00 49 00 00
            // 
            throw new IllegalStateException("An error occurred while decompiling this method.");
        }
        
        final HashMap.TreeNode<K, V> getTreeNode(int n, Object o) {
            return ((this.parent != null) ? this.root() : this).find(n, o, null);
        }
        
        static int tieBreakOrder(Object o, Object o2) {
            int compareTo;
            if (o == null || o2 == null || (compareTo = o.getClass().getName().compareTo(o2.getClass().getName())) == 0) {
                compareTo = ((System.identityHashCode(o) <= System.identityHashCode(o2)) ? -1 : 1);
            }
            return compareTo;
        }
        
        final void treeify(HashMap.Node<K, V>[] array) {
            HashMap.TreeNode balanceInsertion = null;
            while (this != null) {
                HashMap.TreeNode<Object, Object> treeNode = (HashMap.TreeNode)this.next;
                HashMap.TreeNode treeNode2 = this;
                HashMap.TreeNode treeNode3 = this;
                HashMap.TreeNode<K, V> treeNode4 = null;
                treeNode3.right = treeNode4;
                treeNode2.left = treeNode4;
                if (balanceInsertion == null) {
                    this.parent = null;
                    this.red = false;
                    balanceInsertion = this;
                }
                else {
                    Object key = this.key;
                    int hash = this.hash;
                    Class<Object> comparableClass = null;
                    int n;
                    while (true) {
                        Object key2 = balanceInsertion.key;
                        int hash2;
                        if ((hash2 = balanceInsertion.hash) > hash) {
                            n = -1;
                        }
                        else if (hash2 < hash) {
                            n = 1;
                        }
                        else if ((comparableClass == null && (comparableClass = HashMap.comparableClassFor(key)) == null) || (n = HashMap.compareComparables(comparableClass, key, key2)) == 0) {
                            n = tieBreakOrder(key, key2);
                        }
                        if ((balanceInsertion = ((n <= 0) ? balanceInsertion.left : balanceInsertion.right)) == null) {
                            break;
                        }
                        continue;
                    }
                    this.parent = balanceInsertion;
                    if (n <= 0) {
                        balanceInsertion.left = this;
                    }
                    else {
                        balanceInsertion.right = this;
                    }
                    balanceInsertion = balanceInsertion(balanceInsertion, this);
                }
                this = treeNode;
            }
            moveRootToFront(array, balanceInsertion);
        }
        
        final HashMap.Node<K, V> untreeify(HashMap<K, V> hashMap) {
            HashMap.Node<Object, Object> node = null;
            HashMap.Node<Object, Object> node2 = null;
            while (this != null) {
                HashMap.Node<K, V> replacementNode = hashMap.replacementNode(this, null);
                if (node2 == null) {
                    node = replacementNode;
                }
                else {
                    node2.next = replacementNode;
                }
                node2 = replacementNode;
                this = super.next;
            }
            return node;
        }
        
        final HashMap.TreeNode<K, V> putTreeVal(HashMap<K, V> hashMap, HashMap.Node<K, V>[] array, int n, K k, V v) {
            Class<Object> comparableClass = null;
            int n2 = 0;
            this = (this = ((this.parent != null) ? this.root() : this));
            while (true) {
                int hash;
                int n3;
                if ((hash = this.hash) > n) {
                    n3 = -1;
                }
                else if (hash < n) {
                    n3 = 1;
                }
                else {
                    Object key;
                    if ((key = this.key) == k || (k != null && k.equals(key))) {
                        return this;
                    }
                    if ((comparableClass == null && (comparableClass = HashMap.comparableClassFor(k)) == null) || (n3 = HashMap.compareComparables(comparableClass, k, key)) == 0) {
                        if (n2 == 0) {
                            n2 = 1;
                            HashMap.TreeNode<Object, Object> left;
                            HashMap.TreeNode<Object, Object> treeNode;
                            HashMap.TreeNode<Object, Object> right;
                            if (((left = this.left) != null && (treeNode = left.find(n, k, comparableClass)) != null) || ((right = this.right) != null && (treeNode = right.find(n, k, comparableClass)) != null)) {
                                return treeNode;
                            }
                        }
                        n3 = tieBreakOrder(k, key);
                    }
                }
                if ((this = ((n3 <= 0) ? this.left : this.right)) == null) {
                    HashMap.Node<K, V> next = this.next;
                    HashMap.TreeNode<Object, Object> treeNode2 = hashMap.newTreeNode(n, k, v, next);
                    if (n3 <= 0) {
                        this.left = treeNode2;
                    }
                    else {
                        this.right = treeNode2;
                    }
                    this.next = treeNode2;
                    HashMap.TreeNode<K, V> treeNode3 = treeNode2;
                    HashMap.TreeNode treeNode4 = this;
                    treeNode2.prev = treeNode4;
                    treeNode3.parent = treeNode4;
                    if (next != null) {
                        ((HashMap.TreeNode)next).prev = treeNode2;
                    }
                    moveRootToFront(array, balanceInsertion(this, treeNode2));
                    return null;
                }
                else {
                    continue;
                }
            }
        }
        
        final void removeTreeNode(HashMap<K, V> hashMap, HashMap.Node<K, V>[] array, boolean b) {
            int length;
            if (array == null || (length = array.length) == 0) {
                return;
            }
            int n = length - 1 & this.hash;
            HashMap.TreeNode<Object, Object> treeNode = (HashMap.TreeNode)array[n];
            HashMap.TreeNode<Object, Object> next;
            HashMap.TreeNode<K, V> root = next;
            next = (HashMap.TreeNode)this.next;
            HashMap.TreeNode<Object, Object> prev = this.prev;
            if (prev == null) {
                array[n] = next;
            }
            else {
                prev.next = next;
            }
            if (next != null) {
                next.prev = prev;
            }
            if (next == null) {
                return;
            }
            if (root.parent != null) {
                root = root.root();
            }
            HashMap.TreeNode<Object, Object> left;
            if (root == null || (b && (root.right == null || (left = root.left) == null || left.left == null))) {
                array[n] = next.untreeify(hashMap);
                return;
            }
            HashMap.TreeNode<Object, Object> left2 = this.left;
            HashMap.TreeNode<Object, Object> right = this.right;
            HashMap.TreeNode<K, V> right2;
            if (left2 != null && right != null) {
                HashMap.TreeNode<Object, Object> left3;
                for (; (left3 = right.left) != null; right = left3) {}
                int red = right.red;
                right.red = this.red;
                this.red = red;
                right2 = right.right;
                HashMap.TreeNode<Object, Object> parent = this.parent;
                if (right == right) {
                    this.parent = right;
                    right.right = this;
                }
                else {
                    HashMap.TreeNode<Object, Object> parent2 = right.parent;
                    if ((this.parent = parent2) != null) {
                        if (right == parent2.left) {
                            parent2.left = this;
                        }
                        else {
                            parent2.right = this;
                        }
                    }
                    if ((right.right = right) != null) {
                        right.parent = right;
                    }
                }
                this.left = null;
                if ((this.right = right2) != null) {
                    right2.parent = this;
                }
                if ((right.left = left2) != null) {
                    left2.parent = right;
                }
                if ((right.parent = parent) == null) {
                    root = right;
                }
                else if (this == parent.left) {
                    parent.left = right;
                }
                else {
                    parent.right = right;
                }
                if (right2 != null) {
                }
                else {
                    right2 = this;
                }
            }
            else if (left2 != null) {
                right2 = left2;
            }
            else if (right != null) {
                right2 = right;
            }
            else {
                right2 = this;
            }
            if (right2 != this) {
                HashMap.TreeNode<K, V> parent3 = this.parent;
                right2.parent = parent3;
                HashMap.TreeNode<K, V> treeNode2 = parent3;
                if (treeNode2 == null) {
                    root = right2;
                }
                else if (this == treeNode2.left) {
                    treeNode2.left = right2;
                }
                else {
                    treeNode2.right = right2;
                }
                HashMap.TreeNode treeNode3 = this;
                HashMap.TreeNode treeNode4 = this;
                HashMap.TreeNode treeNode5 = this;
                HashMap.TreeNode<K, V> left4 = null;
                treeNode5.parent = left4;
                treeNode4.right = left4;
                treeNode3.left = left4;
            }
            HashMap.TreeNode<K, V> treeNode6 = this.red ? root : balanceDeletion(root, right2);
            if (right2 == this) {
                HashMap.TreeNode<Object, Object> parent4 = this.parent;
                this.parent = null;
                if (parent4 != null) {
                    if (this == parent4.left) {
                        parent4.left = null;
                    }
                    else if (this == parent4.right) {
                        parent4.right = null;
                    }
                }
            }
            if (b) {
                moveRootToFront(array, treeNode6);
            }
            return;
        }
        
        final void split(HashMap<K, V> hashMap, HashMap.Node<K, V>[] array, int n, int n2) {
            HashMap.TreeNode treeNode = null;
            HashMap.TreeNode prev = null;
            HashMap.TreeNode treeNode2 = null;
            HashMap.TreeNode prev2 = null;
            int n3 = 0;
            int n4 = 0;
            while (this != null) {
                HashMap.TreeNode<Object, Object> treeNode3 = (HashMap.TreeNode)this.next;
                this.next = null;
                if ((this.hash & n2) == 0x0) {
                    if ((this.prev = prev) == null) {
                        treeNode = this;
                    }
                    else {
                        prev.next = this;
                    }
                    prev = this;
                    ++n3;
                }
                else {
                    if ((this.prev = prev2) == null) {
                        treeNode2 = this;
                    }
                    else {
                        prev2.next = this;
                    }
                    prev2 = this;
                    ++n4;
                }
                this = treeNode3;
            }
            if (treeNode != null) {
                if (n3 <= 6) {
                    array[n] = treeNode.untreeify(hashMap);
                }
                else {
                    array[n] = treeNode;
                    if (treeNode2 != null) {
                        treeNode.treeify(array);
                    }
                }
            }
            if (treeNode2 != null) {
                if (n4 <= 6) {
                    array[n + n2] = treeNode2.untreeify(hashMap);
                }
                else {
                    array[n + n2] = treeNode2;
                    if (treeNode != null) {
                        treeNode2.treeify(array);
                    }
                }
            }
        }
        
        static <K, V> HashMap.TreeNode<K, V> rotateLeft(HashMap.TreeNode<K, V> treeNode, HashMap.TreeNode<K, V> treeNode2) {
            HashMap.TreeNode<Object, Object> right;
            if (treeNode2 != null && (right = treeNode2.right) != null) {
                HashMap.TreeNode<K, V> left = right.left;
                treeNode2.right = left;
                HashMap.TreeNode<K, V> treeNode3;
                if ((treeNode3 = left) != null) {
                    treeNode3.parent = treeNode2;
                }
                HashMap.TreeNode<K, V> parent = treeNode2.parent;
                right.parent = parent;
                HashMap.TreeNode<K, V> treeNode4;
                if ((treeNode4 = parent) == null) {
                    right.red = false;
                }
                else if (treeNode4.left == treeNode2) {
                    treeNode4.left = right;
                }
                else {
                    treeNode4.right = right;
                }
                right.left = treeNode2;
                treeNode2.parent = right;
                return right;
            }
            else {
                return right;
            }
        }
        
        static <K, V> HashMap.TreeNode<K, V> rotateRight(HashMap.TreeNode<K, V> treeNode, HashMap.TreeNode<K, V> treeNode2) {
            HashMap.TreeNode<Object, Object> left;
            if (treeNode2 != null && (left = treeNode2.left) != null) {
                HashMap.TreeNode<K, V> right = left.right;
                treeNode2.left = right;
                HashMap.TreeNode<K, V> treeNode3;
                if ((treeNode3 = right) != null) {
                    treeNode3.parent = treeNode2;
                }
                HashMap.TreeNode<K, V> parent = treeNode2.parent;
                left.parent = parent;
                HashMap.TreeNode<K, V> treeNode4;
                if ((treeNode4 = parent) == null) {
                    left.red = false;
                }
                else if (treeNode4.right == treeNode2) {
                    treeNode4.right = left;
                }
                else {
                    treeNode4.left = left;
                }
                left.right = treeNode2;
                treeNode2.parent = left;
                return left;
            }
            else {
                return left;
            }
        }
        
        static <K, V> HashMap.TreeNode<K, V> balanceInsertion(HashMap.TreeNode<K, V> treeNode, HashMap.TreeNode<K, V> treeNode2) {
            HashMap.TreeNode<Object, Object> treeNode3;
            treeNode3.red = true;
            while ((treeNode3 = treeNode3.parent) != null) {
                HashMap.TreeNode<Object, Object> parent;
                if (!treeNode3.red || (parent = treeNode3.parent) == null) {
                    return treeNode;
                }
                HashMap.TreeNode<Object, Object> left;
                if (treeNode3 == (left = parent.left)) {
                    HashMap.TreeNode<Object, Object> right;
                    if ((right = parent.right) != null && right.red) {
                        right.red = false;
                        treeNode3.red = false;
                        parent.red = true;
                        treeNode2 = parent;
                    }
                    else {
                        if (treeNode3 == treeNode3.right) {
                            treeNode = rotateLeft(treeNode, treeNode3);
                            parent = (((treeNode3 = treeNode3.parent) == null) ? null : treeNode3.parent);
                        }
                        if (treeNode3 == null) {
                            continue;
                        }
                        treeNode3.red = false;
                        if (parent == null) {
                            continue;
                        }
                        parent.red = true;
                        treeNode = rotateRight(treeNode, parent);
                    }
                }
                else if (left != null && left.red) {
                    left.red = false;
                    treeNode3.red = false;
                    parent.red = true;
                    treeNode2 = parent;
                }
                else {
                    if (treeNode3 == treeNode3.left) {
                        treeNode = rotateRight(treeNode, treeNode3);
                        parent = (((treeNode3 = treeNode3.parent) == null) ? null : treeNode3.parent);
                    }
                    if (treeNode3 == null) {
                        continue;
                    }
                    treeNode3.red = false;
                    if (parent == null) {
                        continue;
                    }
                    parent.red = true;
                    treeNode = rotateLeft(treeNode, parent);
                }
            }
            treeNode3.red = false;
            return treeNode3;
        }
        
        static <K, V> HashMap.TreeNode<K, V> balanceDeletion(HashMap.TreeNode<K, V> treeNode, HashMap.TreeNode<K, V> treeNode2) {
            while (treeNode2 != null && treeNode2 != treeNode) {
                HashMap.TreeNode<Object, Object> treeNode3;
                if ((treeNode3 = treeNode2.parent) == null) {
                    treeNode2.red = false;
                    return treeNode2;
                }
                if (treeNode2.red) {
                    treeNode2.red = false;
                    return treeNode;
                }
                HashMap.TreeNode<Object, Object> left;
                if ((left = treeNode3.left) == treeNode2) {
                    HashMap.TreeNode<Object, Object> right2;
                    if ((right2 = treeNode3.right) != null && right2.red) {
                        right2.red = false;
                        treeNode3.red = true;
                        treeNode = rotateLeft(treeNode, treeNode3);
                        right2 = (((treeNode3 = treeNode2.parent) == null) ? null : treeNode3.right);
                    }
                    if (right2 == null) {
                        treeNode2 = treeNode3;
                    }
                    else {
                        HashMap.TreeNode<Object, Object> left4 = right2.left;
                        HashMap.TreeNode<Object, Object> right3 = right2.right;
                        if ((right3 == null || !right3.red) && (left4 == null || !left4.red)) {
                            right2.red = true;
                            treeNode2 = treeNode3;
                        }
                        else {
                            if (right3 == null || !right3.red) {
                                if (left4 != null) {
                                    left4.red = false;
                                }
                                right2.red = true;
                                treeNode = rotateRight(treeNode, right2);
                                right2 = (((treeNode3 = treeNode2.parent) == null) ? null : treeNode3.right);
                            }
                            if (right2 != null) {
                                right2.red = (treeNode3 != null && treeNode3.red);
                                HashMap.TreeNode<Object, Object> right4;
                                if ((right4 = right2.right) != null) {
                                    right4.red = false;
                                }
                            }
                            if (treeNode3 != null) {
                                treeNode3.red = false;
                                treeNode = rotateLeft(treeNode, treeNode3);
                            }
                            treeNode2 = treeNode;
                        }
                    }
                }
                else {
                    if (left != null && left.red) {
                        left.red = false;
                        treeNode3.red = true;
                        treeNode = rotateRight(treeNode, treeNode3);
                        left = (((treeNode3 = treeNode2.parent) == null) ? null : treeNode3.left);
                    }
                    if (left == null) {
                        treeNode2 = treeNode3;
                    }
                    else {
                        HashMap.TreeNode<Object, Object> left2 = left.left;
                        HashMap.TreeNode<Object, Object> right = left.right;
                        if ((left2 == null || !left2.red) && (right == null || !right.red)) {
                            left.red = true;
                            treeNode2 = treeNode3;
                        }
                        else {
                            if (left2 == null || !left2.red) {
                                if (right != null) {
                                    right.red = false;
                                }
                                left.red = true;
                                treeNode = rotateLeft(treeNode, left);
                                left = (((treeNode3 = treeNode2.parent) == null) ? null : treeNode3.left);
                            }
                            if (left != null) {
                                left.red = (treeNode3 != null && treeNode3.red);
                                HashMap.TreeNode<Object, Object> left3;
                                if ((left3 = left.left) != null) {
                                    left3.red = false;
                                }
                            }
                            if (treeNode3 != null) {
                                treeNode3.red = false;
                                treeNode = rotateRight(treeNode, treeNode3);
                            }
                            treeNode2 = treeNode;
                        }
                    }
                }
            }
            return treeNode;
        }
        
        static <K, V> boolean checkInvariants(HashMap.TreeNode<K, V> treeNode) {
            HashMap.TreeNode<Object, Object> parent = treeNode.parent;
            HashMap.TreeNode<Object, Object> left = treeNode.left;
            HashMap.TreeNode<Object, Object> right = treeNode.right;
            HashMap.TreeNode<Object, Object> prev = treeNode.prev;
            HashMap.TreeNode<Object, Object> treeNode2 = (HashMap.TreeNode)treeNode.next;
            return (prev == null || prev.next == treeNode) && (treeNode2 == null || treeNode2.prev == treeNode) && (parent == null || treeNode == parent.left || treeNode == parent.right) && (left == null || (left.parent == treeNode && left.hash <= treeNode.hash)) && (right == null || (right.parent == treeNode && right.hash >= treeNode.hash)) && (!treeNode.red || left == null || !left.red || right == null || !right.red) && (left == null || checkInvariants(left)) && (right == null || checkInvariants(right));
        }
    }
}
